@@ -75,8 +75,8 @@ pub trait PollingErrorPolicy: Send + Sync + std::fmt::Debug {
         _loop_start: std::time::Instant,
         _attempt_count: u32,
         _operation_name: &str,
-    ) -> Option<Error> {
-        None
+    ) -> Result<(), Error> {
+        Ok(())
     }
 }
 
@@ -321,12 +321,16 @@ where
         }
     }
 
-    fn in_progress_impl(&self, start: std::time::Instant, operation_name: &str) -> Option<Error> {
+    fn in_progress_impl(
+        &self,
+        start: std::time::Instant,
+        operation_name: &str,
+    ) -> Result<(), Error> {
         let now = std::time::Instant::now();
         if now < start + self.maximum_duration {
-            return None;
+            return Ok(());
         }
-        Some(Error::exhausted(Exhausted::new(
+        Err(Error::exhausted(Exhausted::new(
             operation_name,
             "elapsed time",
             format!("{:?}", now.checked_duration_since(start).unwrap()),
@@ -358,10 +362,10 @@ where
         start: std::time::Instant,
         count: u32,
         operation_name: &str,
-    ) -> Option<Error> {
+    ) -> Result<(), Error> {
         self.inner
             .on_in_progress(start, count, operation_name)
-            .or_else(|| self.in_progress_impl(start, operation_name))
+            .and_then(|_| self.in_progress_impl(start, operation_name))
     }
 }
 
@@ -435,11 +439,11 @@ where
         }
     }
 
-    fn in_progress_impl(&self, count: u32, operation_name: &str) -> Option<Error> {
+    fn in_progress_impl(&self, count: u32, operation_name: &str) -> Result<(), Error> {
         if count < self.maximum_attempts {
-            return None;
+            return Ok(());
         }
-        Some(Error::exhausted(Exhausted::new(
+        Err(Error::exhausted(Exhausted::new(
             operation_name,
             "attempt count",
             count.to_string(),
@@ -471,10 +475,10 @@ where
         start: std::time::Instant,
         count: u32,
         operation_name: &str,
-    ) -> Option<Error> {
+    ) -> Result<(), Error> {
         self.inner
             .on_in_progress(start, count, operation_name)
-            .or_else(|| self.in_progress_impl(count, operation_name))
+            .and_then(|_| self.in_progress_impl(count, operation_name))
     }
 }
 
@@ -528,7 +532,7 @@ mod tests {
         Policy {}
         impl PollingErrorPolicy for Policy {
             fn on_error(&self, loop_start: std::time::Instant, attempt_count: u32, error: Error) -> RetryResult;
-            fn on_in_progress(&self, loop_start: std::time::Instant, attempt_count: u32, operation_name: &str) -> Option<Error>;
+            fn on_in_progress(&self, loop_start: std::time::Instant, attempt_count: u32, operation_name: &str) -> Result<(), Error>;
         }
     }
 
@@ -547,7 +551,7 @@ mod tests {
         let p = Aip194Strict;
 
         let now = std::time::Instant::now();
-        assert!(p.on_in_progress(now, 0, "unused").is_none());
+        assert!(p.on_in_progress(now, 0, "unused").is_ok());
         assert!(p.on_error(now, 0, unavailable()).is_continue());
         assert!(p.on_error(now, 0, permission_denied()).is_permanent());
         assert!(p.on_error(now, 0, http_unavailable()).is_continue());
@@ -578,7 +582,7 @@ mod tests {
         let p = AlwaysContinue;
 
         let now = std::time::Instant::now();
-        assert!(p.on_in_progress(now, 0, "unused").is_none());
+        assert!(p.on_in_progress(now, 0, "unused").is_ok());
         assert!(p.on_error(now, 0, http_unavailable()).is_continue());
         assert!(p.on_error(now, 0, unavailable()).is_continue());
     }
@@ -687,15 +691,15 @@ mod tests {
     #[test]
     fn test_limited_elapsed_time_in_progress() {
         let policy = LimitedElapsedTime::new(Duration::from_secs(20));
-        let err = policy.on_in_progress(Instant::now() - Duration::from_secs(10), 1, "unused");
-        assert!(err.is_none(), "{err:?}");
+        let result = policy.on_in_progress(Instant::now() - Duration::from_secs(10), 1, "unused");
+        assert!(result.is_ok(), "{result:?}");
         let err = policy
             .on_in_progress(
                 Instant::now() - Duration::from_secs(30),
                 1,
                 "test-operation-name",
             )
-            .unwrap();
+            .unwrap_err();
         let exhausted = err.source().and_then(|e| e.downcast_ref::<Exhausted>());
         assert!(exhausted.is_some());
     }
@@ -718,13 +722,13 @@ mod tests {
         let mut mock = MockPolicy::new();
         mock.expect_on_in_progress()
             .times(3)
-            .returning(|_, _, _| None);
+            .returning(|_, _, _| Ok(()));
 
         let now = std::time::Instant::now();
         let policy = LimitedElapsedTime::custom(mock, Duration::from_secs(60));
-        assert!(policy.on_in_progress(now, 1, "test-op-name").is_none());
-        assert!(policy.on_in_progress(now, 2, "test-op-name").is_none());
-        assert!(policy.on_in_progress(now, 3, "test-op-name").is_none());
+        assert!(policy.on_in_progress(now, 1, "test-op-name").is_ok());
+        assert!(policy.on_in_progress(now, 2, "test-op-name").is_ok());
+        assert!(policy.on_in_progress(now, 3, "test-op-name").is_ok());
     }
 
     #[test]
@@ -732,11 +736,11 @@ mod tests {
         let mut mock = MockPolicy::new();
         mock.expect_on_in_progress()
             .times(1)
-            .returning(|_, _, _| Some(transient_error()));
+            .returning(|_, _, _| Err(transient_error()));
 
         let now = std::time::Instant::now();
         let policy = LimitedElapsedTime::custom(mock, Duration::from_secs(60));
-        assert!(policy.on_in_progress(now, 1, "test-op-name").is_some());
+        assert!(policy.on_in_progress(now, 1, "test-op-name").is_err());
     }
 
     #[test]
@@ -809,11 +813,11 @@ mod tests {
     #[test]
     fn test_limited_attempt_count_in_progress() {
         let policy = LimitedAttemptCount::new(20);
-        let err = policy.on_in_progress(Instant::now(), 10, "unused");
-        assert!(err.is_none(), "{err:?}");
+        let result = policy.on_in_progress(Instant::now(), 10, "unused");
+        assert!(result.is_ok(), "{result:?}");
         let err = policy
             .on_in_progress(Instant::now(), 30, "test-operation-name")
-            .unwrap();
+            .unwrap_err();
         let exhausted = err.source().and_then(|e| e.downcast_ref::<Exhausted>());
         assert!(exhausted.is_some());
     }
@@ -837,13 +841,13 @@ mod tests {
         let mut mock = MockPolicy::new();
         mock.expect_on_in_progress()
             .times(3)
-            .returning(|_, _, _| None);
+            .returning(|_, _, _| Ok(()));
 
         let now = std::time::Instant::now();
         let policy = LimitedAttemptCount::custom(mock, 5);
-        assert!(policy.on_in_progress(now, 1, "test-op-name").is_none());
-        assert!(policy.on_in_progress(now, 2, "test-op-name").is_none());
-        assert!(policy.on_in_progress(now, 3, "test-op-name").is_none());
+        assert!(policy.on_in_progress(now, 1, "test-op-name").is_ok());
+        assert!(policy.on_in_progress(now, 2, "test-op-name").is_ok());
+        assert!(policy.on_in_progress(now, 3, "test-op-name").is_ok());
     }
 
     #[test]
@@ -851,11 +855,11 @@ mod tests {
         let mut mock = MockPolicy::new();
         mock.expect_on_in_progress()
             .times(1)
-            .returning(|_, _, _| Some(unavailable()));
+            .returning(|_, _, _| Err(unavailable()));
 
         let now = std::time::Instant::now();
         let policy = LimitedAttemptCount::custom(mock, 5);
-        assert!(policy.on_in_progress(now, 1, "test-op-name").is_some());
+        assert!(policy.on_in_progress(now, 1, "test-op-name").is_err());
     }
 
     #[test]
