@@ -290,42 +290,49 @@ impl LinuxMenu {
     }
 
     fn command(&self) -> Result<String, MenuInstError> {
-        let mut parts = Vec::new();
-        if let Some(pre_command) = &self.command.precommand {
-            parts.push(pre_command.resolve(&self.placeholders));
-        }
+        let envs = self
+            .command
+            .activate
+            .unwrap_or(false)
+            .then(|| {
+                // create a bash activation script and emit it into the script
+                let activator =
+                    Activator::from_path(&self.prefix, shell::Bash, Platform::current())?;
+                let activation_variables = ActivationVariables {
+                    path_modification_behavior: PathModificationBehavior::Prepend,
+                    ..Default::default()
+                };
+                let activation_env = activator.run_activation(activation_variables, None)?;
 
-        let mut envs = Vec::new();
-        if self.command.activate.unwrap_or(false) {
-            // create a bash activation script and emit it into the script
-            let activator = Activator::from_path(&self.prefix, shell::Bash, Platform::current())?;
-            let activation_variables = ActivationVariables {
-                path_modification_behavior: PathModificationBehavior::Prepend,
-                ..Default::default()
-            };
-            let activation_env = activator.run_activation(activation_variables, None)?;
+                let envs = activation_env
+                    .into_iter()
+                    .map(|(k, v)| format!(r#""{k}={v}""#))
+                    .collect();
+                Ok::<Vec<String>, MenuInstError>(envs)
+            })
+            .transpose()?
+            .unwrap_or_default();
 
-            for (k, v) in activation_env {
-                envs.push(format!(r#"{k}="{v}""#));
-            }
-        }
-
-        let command = self
+        let main_command = self
             .command
             .command
             .iter()
             .map(|s| s.resolve(&self.placeholders))
-            .collect::<Vec<_>>()
+            .map(|part| Ok(shlex::try_quote(&part)?.into_owned()))
+            .collect::<Result<Vec<_>, MenuInstError>>()?
             .join(" ");
 
-        parts.push(command);
-
-        let command = parts.join(" && ");
-        let bash_command = format!("bash -c {}", shlex::try_quote(&command)?);
-        if envs.is_empty() {
-            Ok(bash_command)
+        let main_with_env = if envs.is_empty() {
+            main_command
         } else {
-            Ok(format!("env {} {}", envs.join(" "), bash_command))
+            format!("env {} {}", envs.join(" "), main_command)
+        };
+
+        if let Some(pre_command) = &self.command.precommand {
+            let command = [pre_command.0.clone(), main_with_env].join(" && ");
+            Ok(format!("bash -c {}", shlex::try_quote(&command)?))
+        } else {
+            Ok(main_with_env)
         }
     }
 
@@ -654,7 +661,7 @@ mod tests {
     };
     use tempfile::TempDir;
 
-    use crate::{schema::MenuInstSchema, test::test_data};
+    use crate::{render::PlaceholderString, schema::MenuInstSchema, test::test_data};
 
     use super::{Directories, LinuxMenu};
 
@@ -816,4 +823,133 @@ mod tests {
         let mime_file_content = fs::read_to_string(&mime_file).unwrap();
         insta::assert_snapshot!(mime_file_content);
     }
+
+    #[test]
+    fn test_command_without_activation() {
+        let dirs = FakeDirectories::new();
+        let fake_prefix = FakePrefix::new("spyder/menu.json");
+
+        let item = fake_prefix.schema.menu_items[0].clone();
+        let linux = item.platforms.linux.unwrap();
+        let mut command = item.command.merge(linux.base);
+        command.activate = Some(false);
+
+        let placeholders = super::BaseMenuItemPlaceholders::new(
+            fake_prefix.prefix(),
+            fake_prefix.prefix(),
+            rattler_conda_types::Platform::current(),
+        );
+
+        let linux_menu = LinuxMenu::new_with_directories(
+            &fake_prefix.schema.menu_name,
+            fake_prefix.prefix(),
+            linux.specific,
+            command,
+            &placeholders,
+            dirs.directories().clone(),
+        );
+
+        let result = linux_menu.command().unwrap();
+        let normalized_result = result.replace(fake_prefix.prefix().to_str().unwrap(), "<PREFIX>");
+        insta::assert_snapshot!(normalized_result, @"<PREFIX>/bin/spyder '%F'");
+    }
+
+    #[test]
+    fn test_command_with_activation() {
+        let dirs = FakeDirectories::new();
+        let fake_prefix = FakePrefix::new("spyder/menu.json");
+
+        let item = fake_prefix.schema.menu_items[0].clone();
+        let linux = item.platforms.linux.unwrap();
+        let mut command = item.command.merge(linux.base);
+        command.activate = Some(true);
+
+        let placeholders = super::BaseMenuItemPlaceholders::new(
+            fake_prefix.prefix(),
+            fake_prefix.prefix(),
+            rattler_conda_types::Platform::current(),
+        );
+
+        let linux_menu = LinuxMenu::new_with_directories(
+            &fake_prefix.schema.menu_name,
+            fake_prefix.prefix(),
+            linux.specific,
+            command,
+            &placeholders,
+            dirs.directories().clone(),
+        );
+
+        let result = linux_menu.command().unwrap();
+        assert!(result.starts_with("env "));
+        assert!(result.contains("CONDA_PREFIX="));
+        assert!(result.contains("CONDA_SHLVL="));
+        assert!(result.contains("PATH="));
+        assert!(result.contains("spyder"));
+    }
+
+    #[test]
+    fn test_command_with_precommand() {
+        let dirs = FakeDirectories::new();
+        let fake_prefix = FakePrefix::new("spyder/menu.json");
+
+        let item = fake_prefix.schema.menu_items[0].clone();
+        let linux = item.platforms.linux.unwrap();
+        let mut command = item.command.merge(linux.base);
+        command.precommand = Some(PlaceholderString("echo 'setup'".to_string()));
+        command.activate = Some(false);
+
+        let placeholders = super::BaseMenuItemPlaceholders::new(
+            fake_prefix.prefix(),
+            fake_prefix.prefix(),
+            rattler_conda_types::Platform::current(),
+        );
+
+        let linux_menu = LinuxMenu::new_with_directories(
+            &fake_prefix.schema.menu_name,
+            fake_prefix.prefix(),
+            linux.specific,
+            command,
+            &placeholders,
+            dirs.directories().clone(),
+        );
+
+        let result = linux_menu.command().unwrap();
+        let normalized_result = result.replace(fake_prefix.prefix().to_str().unwrap(), "<PREFIX>");
+        insta::assert_snapshot!(normalized_result, @r#"bash -c "echo 'setup' && <PREFIX>/bin/spyder '%F'""#);
+    }
+
+    #[test]
+    fn test_command_with_complex_arguments() {
+        let dirs = FakeDirectories::new();
+        let fake_prefix = FakePrefix::new("spyder/menu.json");
+
+        let item = fake_prefix.schema.menu_items[0].clone();
+        let linux = item.platforms.linux.unwrap();
+        let mut command = item.command.merge(linux.base);
+        command.command = vec![
+            PlaceholderString("python".to_string()),
+            PlaceholderString("-m".to_string()),
+            PlaceholderString("spyder".to_string()),
+            PlaceholderString("--arg with spaces".to_string()),
+        ];
+        command.activate = Some(false);
+
+        let placeholders = super::BaseMenuItemPlaceholders::new(
+            fake_prefix.prefix(),
+            fake_prefix.prefix(),
+            rattler_conda_types::Platform::current(),
+        );
+
+        let linux_menu = LinuxMenu::new_with_directories(
+            &fake_prefix.schema.menu_name,
+            fake_prefix.prefix(),
+            linux.specific,
+            command,
+            &placeholders,
+            dirs.directories().clone(),
+        );
+
+        let result = linux_menu.command().unwrap();
+        insta::assert_snapshot!(result, @"python -m spyder '--arg with spaces'");
+    }
 }
