@@ -18,8 +18,11 @@ use url::Url;
 
 use super::ShardedRepodata;
 use crate::{
-    fetch::CacheAction, gateway::sharded_subdir::decode_zst_bytes_async,
-    reporter::ResponseReporterExt, utils::url_to_cache_filename, GatewayError, Reporter,
+    fetch::CacheAction,
+    gateway::sharded_subdir::decode_zst_bytes_async,
+    reporter::{DownloadReporter, ResponseReporterExt},
+    utils::url_to_cache_filename,
+    GatewayError, Reporter,
 };
 
 const REPODATA_SHARDS_FILENAME: &str = "repodata_shards.msgpack.zst";
@@ -38,7 +41,7 @@ pub async fn fetch_index(
         cache_path: &Path,
         policy: CachePolicy,
         response: Response,
-        reporter: Option<(&dyn Reporter, usize)>,
+        reporter: Option<(&dyn DownloadReporter, usize)>,
         permit: Option<tokio::sync::SemaphorePermit<'_>>,
     ) -> Result<ShardedRepodata, GatewayError> {
         let response = response.error_for_status()?;
@@ -185,8 +188,9 @@ pub async fn fetch_index(
                         .expect("failed to acquire semaphore permit");
 
                         // Send the request
-                        let download_reporter =
-                            reporter.map(|r| (r, r.on_download_start(&shards_url)));
+                        let download_reporter = reporter
+                            .and_then(Reporter::download_reporter)
+                            .map(|r| (r, r.on_download_start(&shards_url)));
                         let response = client.execute(request).await?;
 
                         match cache_header.policy.after_response(
@@ -266,7 +270,9 @@ pub async fn fetch_index(
     .expect("failed to acquire semaphore permit");
 
     // Do a fresh requests
-    let reporter = reporter.map(|r| (r, r.on_download_start(&shards_url)));
+    let reporter = reporter
+        .and_then(Reporter::download_reporter)
+        .map(|r| (r, r.on_download_start(&shards_url)));
     let response = client
         .execute(
             request
